import eyekit

sentence = 'The quick brown fox [jump]{stem_1}[ed]{suffix_1} over the lazy dog.'
txt = eyekit.TextBlock(sentence, position=(100, 500), font_face='Times New Roman', font_size=36)
seq = eyekit.FixationSequence([[106, 490, 100], [190, 486, 100], [230, 505, 100], [298, 490, 100], [361, 497, 100], [430, 489, 100], [450, 505, 100], [492, 491, 100], [562, 505, 100], [637, 493, 100], [712, 497, 100], [763, 487, 100]])

def test_Image():
	img = eyekit.vis.Image(1920, 1080)
	img.set_caption('Quick Brown Fox', 'Helvetica Neue italic', 8)
	img.set_background_color('snow')
	img.draw_text_block(txt)
	for word in txt.words():
		img.draw_rectangle(word.box, color='crimson')
	img.draw_fixation_sequence(seq)
	img.draw_line((0, 0), (1920, 1080), color='coral', stroke_width=2, dashed=True)
	img.draw_circle(200, 200, 200)
	img.draw_annotation(1000, 500, 'Hello world!', font_face='Courier New', font_size=26, color='yellowgreen')
	# img.save('private/dump/test_image.pdf')
	fig = eyekit.vis.Figure(1, 2)
	fig.set_padding(vertical=10, horizontal=5, edge=2)
	fig.add_image(img)
	fig.add_image(img)
	# fig.save('private/dump/test_figure.pdf')

def test_mm_to_pts():
	assert str(eyekit.vis._mm_to_pts(1))[:5] == '2.834'
	assert str(eyekit.vis._mm_to_pts(10))[:5] == '28.34'

def test_color_to_rgb():
	assert eyekit.vis._color_to_rgb((255, 0, 0)) == (1.0, 0.0, 0.0)
	assert eyekit.vis._color_to_rgb('#FF0000') == (1.0, 0.0, 0.0)
	assert eyekit.vis._color_to_rgb('red') == (1.0, 0.0, 0.0)


import re
import cairocffi as cairo

regex_italic = re.compile(' italic', re.IGNORECASE)
regex_bold = re.compile(' bold', re.IGNORECASE)

class Font:

	'''

	Wrapper around Cairo's font selection mechanism.

	'''

	def __init__(self, face, size):
		if regex_italic.search(face):
			self.slant = 'italic'
			slant = cairo.FONT_SLANT_ITALIC
			face = regex_italic.sub('', face)
		else:
			self.slant = 'normal'
			slant = cairo.FONT_SLANT_NORMAL
		if regex_bold.search(face):
			self.weight = 'bold'
			weight = cairo.FONT_WEIGHT_BOLD
			face = regex_bold.sub('', face)
		else:
			self.weight = 'normal'
			weight = cairo.FONT_WEIGHT_NORMAL
		self.family = face.strip()
		self.size = float(size)
		
		self.toy_font_face = cairo.ToyFontFace(self.family, slant, weight)
		self.scaled_font = cairo.ScaledFont(self.toy_font_face, cairo.Matrix(xx=self.size, yy=self.size))

	def calculate_width(self, text):
		return self.scaled_font.text_extents(text)[4]
		
	def calculate_height(self, text):
		return self.scaled_font.text_extents(text)[3]



special_characters_by_language = {
	'Bulgarian': 'АБВГДЕЖЗИЙЍКЛМНОПРСТУФХЦЧШЩЪЬЮЯабвгдежзийѝклмнопрстуфхцчшщъьюя',
	'Croatian': 'ČĆĐŠŽčćđšž',
	'Czech': 'ÁČĎÉĚÍŇÓŘŠŤÚŮÝŽáčďéěíňóřšťúůýž',
	'Danish': 'ÆØÅæøå',
	'Estonian': 'ŠŽÕÄÖÜšžõäöü',
	'Finnish': 'ÅÄÖåäö',
	'French': 'ÇÉÀÈÙÂÊÎÔÛËÏÜŸŒçéàèùâêîôûëïüÿœ',
	'German': 'ÄÖÜäöüß',
	'Greek': 'ΑΒΓΔΕΖΗΘΙΚΛΜΝΞΟΠΡΣΤΥΦΧΨΩΆΈΉΊΌΎΏΪΫαβγδεζηθικλμνξοπρσςτυφχψωάέήίόύώϊϋΐΰ',
	'Hungarian': 'ÁÉÍÓÖŐÚÜŰáéíóöőúüű',
	'Irish': 'ÁÉÍÓÚáéíóú',
	'Italian': 'ÀÉÈÍÌÓÒÚÙàéèíìóòúù',
	'Latvian': 'ĀČĒĢĪĶĻŅŠŪŽāčēģīķļņšūž',
	'Lithuanian': 'ĄČĘĖĮŠŲŪŽąčęėįšųūž',
	'Maltese': 'ĊĠĦŻċġħż',
	'Norwegian': 'ÆØÅæøå',
	'Polish': 'ĄĆĘŁŃÓŚŹŻąćęłńóśźż',
	'Portuguese': 'ÁÂÃÀÇÉÊÍÓÔÕÚáâãàçéêíóôõú',
	'Romanian': 'ĂÂÎȘȚăâîșț',
	'Russian': 'АБВГДЕЁЖЗИЙКЛМНОПРСТУФХЦЧШЩЪЫЬЭЮЯабвгдеёжзийклмнопрстуфхцчшщъыьэюя',
	'Slovak': 'ÁÄČĎÉÍĹĽŇÓÔŔŠŤÚÝŽáäčďéíĺľňóôŕšťúýž',
	'Slovene': 'ČŠŽčšž',
	'Spanish': 'ÑÁÉÍÓÚÜñáéíóúü',
	'Swedish': 'ÅÄÖåäö',
	'Turkish': 'ÇĞİÖŞÜçşğıiöü'
}

special_characters = []
for characters in special_characters_by_language.values():
	special_characters.extend(characters)

characters = 'A-Za-z' + ''.join(set(special_characters))



import eyekit

sentence = 'The quick brown fox [jump]{stem_1}[ed]{suffix_1} over the lazy dog.'
txt = eyekit.TextBlock(sentence, position=(100, 500), font_face='Times New Roman', font_size=36)
seq = eyekit.FixationSequence([[106, 490, 100], [190, 486, 100], [230, 505, 100], [298, 490, 100], [361, 497, 100], [430, 489, 100], [450, 505, 100], [492, 491, 100], [562, 505, 100], [637, 493, 100], [712, 497, 100], [763, 487, 100]])

def test_initialization():
	assert txt.position == (100, 500)
	assert txt.font_face == 'Times New Roman'
	assert txt.font_size == 36
	assert txt.line_height == 36
	assert txt.alphabet is None
	assert txt.n_rows == 1
	assert txt.n_cols == 45
	assert len(txt) == 45

def test_zone_extraction():
	assert len(list(txt.zones())) == 2
	for zone in txt.zones():
		assert zone.id in ['stem_1', 'suffix_1']
		assert zone.text in ['jump', 'ed']
		assert zone.baseline == 500
		assert zone.height == 36

def test_word_extraction():
	assert len(list(txt.words())) == 9
	for word in txt.words():
		assert word.text in ['The', 'quick', 'brown', 'fox', 'jumped', 'over', 'the', 'lazy', 'dog']
		assert word.baseline == 500
		assert word.height == 36

def test_arbitrary_extraction():
	assert txt[0:0:3].text == 'The'
	assert txt[0:41:45].text == 'dog.'
	assert txt['0:0:3'].text == 'The'
	assert txt['0:41:45'].text == 'dog.'

def test_which_methods():
	for fixation, answer in zip(seq, ['The','quick','quick','brown','fox','jumped','jumped','jumped','over','the','lazy','dog']):
		assert txt.which_word(fixation).text == answer
	for fixation, answer in zip(seq, ['T', 'u', 'k', 'o', 'f', 'u', 'm', 'e', 'v', 'e', 'y', 'g']):
		assert txt.which_character(fixation).text == answer

def test_serialize():
	data = txt._serialize()
	assert data['text'] == [sentence]
	assert data['position'] == (100, 500)
	assert data['font_face'] == 'Times New Roman'
	assert data['font_size'] == 36
	assert data['line_height'] == 36

def test_analysis_functions():
	results = eyekit.analysis.initial_fixation_duration(txt.zones(), seq)
	assert results['stem_1'] == 100
	assert results['suffix_1'] == 100
	results = eyekit.analysis.total_fixation_duration(txt.zones(), seq)
	assert results['stem_1'] == 200
	assert results['suffix_1'] == 100
	results = eyekit.analysis.gaze_duration(txt.zones(), seq)
	assert results['stem_1'] == 200
	assert results['suffix_1'] == 100
	results = eyekit.analysis.initial_landing_position(txt.zones(), seq)
	assert results['stem_1'] == 2
	assert results['suffix_1'] == 1
	results = eyekit.analysis.initial_landing_x(txt.zones(), seq)
	assert int(results['stem_1']) == 18
	assert int(results['suffix_1']) == 6



import eyekit

data = eyekit.io.read('example/example_data.json')
texts = eyekit.io.read('example/example_texts.json')

def test_read():
	assert isinstance(data['trial_0']['fixations'], eyekit.FixationSequence)
	assert isinstance(data['trial_1']['fixations'], eyekit.FixationSequence)
	assert isinstance(data['trial_2']['fixations'], eyekit.FixationSequence)
	assert data['trial_0']['fixations'][0].x == 412
	assert data['trial_0']['fixations'][1].y == 163
	assert data['trial_0']['fixations'][2].duration == 334
	assert isinstance(texts['passage_a']['text'], eyekit.TextBlock)
	assert isinstance(texts['passage_b']['text'], eyekit.TextBlock)
	assert isinstance(texts['passage_c']['text'], eyekit.TextBlock)
	assert texts['passage_a']['text'].position == (360, 161)
	assert texts['passage_a']['text'].font_face == 'Courier New'



from eyekit import _font

def test_Font():
	font = _font.Font('Helvetica Neue bold italic', 12)
	assert font.family == 'Helvetica Neue'
	assert font.slant == 'italic'
	assert font.weight == 'bold'
	assert font.size == 12